﻿/*
 * Created by SharpDevelop.
 * User: pshai
 * Date: 6.7.2016
 * Time: 10.24
 * 
 * To change this template use Tools | Options | Coding | Edit Standard Headers.
 */

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using TradeServer.ScriptingDriver.DataObjects;
using TradeServer.ScriptingDriver.Interfaces;
using TicTacTec.TA.Library;

namespace Pshais.Indicators
{
	public class SAR : IIndicatorScript
	{
		double acceleration = 0.02;
		double maxStep = 0.2;
		
		string[] colors = {
			"#22FF00",
			"#FF6600"
		};
		
		int activeColor = 0;
		
		double sar = 0;

		#region IIndicatorScript implementation
		public void Init()
		{
			
		}
		
		public IndicatorResult GetResult(IndicatorContext context)
		{			
			try
            {
                IPriceInstrument instrument = context.PriceInstrument;
                int position = instrument.Close.Length - (1 + context.Offset);
                double price = instrument.Close[ position ];
                
                int beginIndex, outNBElements;
                var sarValues = new double[position];
                	
                var sarReturnCode = Core.Sar( 1,
                                              position,
                                              instrument.High,
                                              instrument.Low,
                                              acceleration,
                                              maxStep,
                                              out beginIndex,
                                              out outNBElements,
                                              sarValues
                                              );

                if (sarReturnCode == Core.RetCode.Success)
                {
                    if (outNBElements > 0)
                    {
                    	sar = sarValues[outNBElements-1];
                    }
                }
                
                if( sar <= price )
                {
                	activeColor = 0;
                	return IndicatorResult.BuyLong;
                }
                else
                {
                	activeColor = 1;
                	return IndicatorResult.SellShort;
                }
			}
            catch (Exception ex) // This try catch block is an highly recommend practice
            {
                context.Logger.LogToFile(string.Format("Exception detected: {0}", ex));
            }

            return IndicatorResult.StayNoPosition; // Default output
		}
		
		public List<ScriptParameter> GetParameters()
		{
            var rtn = new List<ScriptParameter>
            {
                new ScriptParameter("Acceleration", ScriptParameterType.Decimal, acceleration.ToString()),
                new ScriptParameter("Max. Step", ScriptParameterType.Decimal, maxStep.ToString())
            };

            return rtn;
		}
		
		public void SetParameters(Dictionary<string, object> parameters)
		{
			acceleration = ParseDouble(parameters["Acceleration"]);
			maxStep = ParseDouble(parameters["Max. Step"]);
		}
		
		public List<ChartRecord> GetChartLines()
		{
            List<ChartRecord> chartLines = new List<ChartRecord>
            {
                new ChartRecord() { ChartIndex = 0, Name = "SAR", HexLineColor = "#FFFFFF" }
            };

            return chartLines;
		}
		
		public List<decimal> GetChartData()
		{
            var chartData = new List<decimal>
            {
                Convert.ToDecimal(sar)
            };

            return chartData;
		}
		
		public string Name {
			get {
				return "pshai's Parabolic SAR";
			}
		}
		
		public int TicksBack {
			get {
				return Core.SarLookback(acceleration, maxStep) + 1;
			}
		}
		#endregion
		
		double ParseDouble(object obj)
		{
			var numStyle = System.Globalization.NumberStyles.Any;
			var cult = System.Globalization.CultureInfo.InvariantCulture;
			
			double output;
			try {
				double.TryParse(
					obj.ToString().Replace(',', '.'),
					numStyle,
					cult,
					out output);
			}
			catch
			{
				output = 0;
			}
			
			return output;
		}
	}
}